"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const markdown_it_1 = __importDefault(require("markdown-it"));
const yamlFront = require('yaml-front-matter');
const path = __importStar(require("path"));
const interfaces_1 = require("./interfaces");
const it = new markdown_it_1.default({});
function isUndefined(meta, key) {
    return meta[key] != null;
}
function fixPrefix(meta, fileName) {
    if (isUndefined(meta, 'prefix')) {
        return meta.prefix;
    }
    const name = fileName.split(path.sep).pop();
    return name.endsWith('.md') ? name.substr(0, name.length - 3) : name;
}
function parse(md, fileName, config) {
    const meta = yamlFront.loadFront(md);
    const tokens = it.parse(meta.__content.toString() || '', {});
    delete meta.__content;
    const codeToken = tokens.find(w => w.tag === 'code');
    if (!codeToken) {
        throw new Error(`Must have a code body`);
    }
    if (!isUndefined(meta, 'description')) {
        const paragraphOpenTokenIndex = tokens.findIndex(w => w.type === 'paragraph_open');
        if (paragraphOpenTokenIndex !== -1 && tokens.length >= (paragraphOpenTokenIndex + 1)) {
            const paragraphToken = tokens[paragraphOpenTokenIndex + 1];
            meta.description = paragraphToken.content.trim();
        }
    }
    else if (typeof meta.description === 'object') {
        const source = meta.description;
        if (typeof config.i18nTpl === 'string' && config.i18nTpl.length > 0) {
            meta.description = config.i18nTpl.replace(/\{([^}]+)\}/g, (word, lang) => {
                return source[lang] || source[interfaces_1.DEFAULT_LANG];
            });
        }
        else {
            meta.description = source[config.i18n] || source[interfaces_1.DEFAULT_LANG];
        }
    }
    meta.prefix = fixPrefix(meta, fileName);
    meta.body = codeToken.content.trim();
    return meta;
}
exports.parse = parse;
